@@ -3,7 +3,7 @@
  * 
  * (Word 2007 format)
  *
- * Copyright (c) Chris Putnam 2007-2013
+ * Copyright (c) Chris Putnam 2007-2021
  *
  * Source code released under the GPL version 2
  *
@@ -11,36 +11,63 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include "newstr.h"
+#include "str.h"
 #include "fields.h"
+#include "month.h"
 #include "utf8.h"
-#include "wordout.h"
+#include "bibformats.h"
 
-void
-wordout_initparams( param *p, const char *progname )
+/*****************************************************
+ PUBLIC: int wordout_initparams()
+*****************************************************/
+
+static void wordout_writeheader( FILE *outptr, param *p );
+static void wordout_writefooter( FILE *outptr );
+static int  wordout_write( fields *info, FILE *outptr, param *p, unsigned long numrefs );
+
+int
+wordout_initparams( param *pm, const char *progname )
 {
-	p->writeformat      = BIBL_WORD2007OUT;
-	p->format_opts      = 0;
-	p->charsetout       = BIBL_CHARSET_UNICODE;
-	p->charsetout_src   = BIBL_SRC_DEFAULT;
-	p->latexout         = 0;
-	p->utf8out          = BIBL_CHARSET_UTF8_DEFAULT;
-	p->utf8bom          = BIBL_CHARSET_BOM_DEFAULT;
-	if ( !p->utf8out ) p->xmlout = 3;
-	else p->xmlout = 1;
-	p->nosplittitle     = 0;
-	p->verbose          = 0;
-	p->addcount         = 0;
-	p->singlerefperfile = 0;
+	pm->writeformat      = BIBL_WORD2007OUT;
+	pm->format_opts      = 0;
+	pm->charsetout       = BIBL_CHARSET_UNICODE;
+	pm->charsetout_src   = BIBL_SRC_DEFAULT;
+	pm->latexout         = 0;
+	pm->utf8out          = BIBL_CHARSET_UTF8_DEFAULT;
+	pm->utf8bom          = BIBL_CHARSET_BOM_DEFAULT;
+	if ( !pm->utf8out )
+		pm->xmlout   = BIBL_XMLOUT_ENTITIES;
+	else
+		pm->xmlout   = BIBL_XMLOUT_TRUE;
+	pm->nosplittitle     = 0;
+	pm->verbose          = 0;
+	pm->addcount         = 0;
+	pm->singlerefperfile = 0;
 
-	p->headerf = wordout_writeheader;
-	p->footerf = wordout_writefooter;
-	p->writef  = wordout_write;
+	pm->headerf   = wordout_writeheader;
+	pm->footerf   = wordout_writefooter;
+	pm->assemblef = NULL;
+	pm->writef    = wordout_write;
+
+	if ( !pm->progname ) {
+		if ( !progname ) pm->progname = NULL;
+		else {
+			pm->progname = strdup( progname );
+			if ( !pm->progname ) return BIBL_ERR_MEMERR;
+		}
+	}
+
+	return BIBL_OK;
 }
 
+/*****************************************************
+ PUBLIC: int wordout_write()
+*****************************************************/
+
 typedef struct convert {
-	char oldtag[25];
-	char newtag[25];
+	char *oldtag;
+	char *newtag;
+	char *prefix;
 	int  code;
 } convert;
 
@@ -96,35 +123,47 @@ enum {
 	TYPE_PHDTHESIS,
 };
 
-/*
- * fixed output
- */
 static void
-output_fixed( FILE *outptr, char *tag, char *data, int level )
+output_level( FILE *outptr, int level )
 {
 	int i;
-	for ( i=0; i<level; ++i ) fprintf( outptr, " " );
-	fprintf( outptr, "<%s>%s</%s>\n", tag, data, tag );
+	for ( i=0; i<level; ++i )
+		fprintf( outptr, " " );
+}
+
+/* fixed output
+ *
+ * <TAG>value</TAG>
+ */
+static void
+output_fixed( FILE *outptr, const char *tag, const char *value, int level )
+{
+	output_level( outptr, level );
+	fprintf( outptr, "<%s>%s</%s>\n", tag, value, tag );
 }
 
 /* detail output
  *
+ * <TAG>value</TAG>
  */
 static void
-output_item( fields *info, FILE *outptr, char *tag, int item, int level )
+output_item( fields *info, FILE *outptr, const char *tag, const char *prefix, int item, int level )
 {
-	int i;
-	if ( item==-1 ) return;
-	for ( i=0; i<level; ++i ) fprintf( outptr, " " );
-	fprintf( outptr, "<%s>%s</%s>\n", tag, info->data[item].data, tag );
-	fields_setused( info, item );
+	if ( item!=-1 ) {
+		output_level( outptr, level );
+		fprintf( outptr, "<%s>%s%s</%s>\n",
+			tag,
+			prefix,
+			(char*) fields_value( info, item, FIELDS_CHRP ),
+			tag
+		);
+	}
 }
 
 static void
-output_itemv( FILE *outptr, char *tag, char *item, int level )
+output_itemv( FILE *outptr, const char *tag, const char *item, int level )
 {
-	int i;
-	for ( i=0; i<level; ++i ) fprintf( outptr, " " );
+	output_level( outptr, level );
 	fprintf( outptr, "<%s>%s</%s>\n", tag, item, tag );
 }
 
@@ -134,19 +173,14 @@ output_itemv( FILE *outptr, char *tag, char *item, int level )
  *
  */
 static void
-output_range( FILE *outptr, char *tag, char *start, char *end, int level )
+output_range( FILE *outptr, const char *tag, const char *start, const char *end, int level )
 {
-	int i;
-	if ( start==NULL && end==NULL ) return;
-	if ( start==NULL )
-		output_itemv( outptr, tag, end, 0 );
-	else if ( end==NULL )
-		output_itemv( outptr, tag, start, 0 );
-	else {
-		for ( i=0; i<level; ++i )
-			fprintf( outptr, " " );
+	if ( start && end ) {
+		output_level( outptr, level );
 		fprintf( outptr, "<%s>%s-%s</%s>\n", tag, start, end, tag );
 	}
+	else if ( start ) output_itemv( outptr, tag, start, level );
+	else if ( end )   output_itemv( outptr, tag, end,   level );
 }
 
 static void
@@ -155,37 +189,38 @@ output_list( fields *info, FILE *outptr, convert *c, int nc )
         int i, n;
         for ( i=0; i<nc; ++i ) {
                 n = fields_find( info, c[i].oldtag, c[i].code );
-                if ( n!=-1 ) output_item( info, outptr, c[i].newtag, n, 0 );
+                if ( n!=FIELDS_NOTFOUND ) output_item( info, outptr, c[i].newtag, c[i].prefix, n, 0 );
         }
 
 }
 
 typedef struct outtype {
 	int value;
-	char *out;
+	const char *out;
 } outtype;
 
 static
 outtype genres[] = {
-	{ TYPE_PATENT, "patent" },
-	{ TYPE_REPORT, "report" },
-	{ TYPE_CASE,   "legal case and case notes" },
-	{ TYPE_ART,    "art original" },
-	{ TYPE_ART,    "art reproduction" },
-	{ TYPE_ART,    "comic strip" },
-	{ TYPE_ART,    "diorama" },
-	{ TYPE_ART,    "graphic" },
-	{ TYPE_ART,    "model" },
-	{ TYPE_ART,    "picture" },
+	{ TYPE_PATENT,           "patent" },
+	{ TYPE_REPORT,           "report" },
+	{ TYPE_REPORT,           "technical report" },
+	{ TYPE_CASE,             "legal case and case notes" },
+	{ TYPE_ART,              "art original" },
+	{ TYPE_ART,              "art reproduction" },
+	{ TYPE_ART,              "comic strip" },
+	{ TYPE_ART,              "diorama" },
+	{ TYPE_ART,              "graphic" },
+	{ TYPE_ART,              "model" },
+	{ TYPE_ART,              "picture" },
 	{ TYPE_ELECTRONICSOURCE, "electronic" },
-	{ TYPE_FILM,   "videorecording" },
-	{ TYPE_FILM,   "motion picture" },
-	{ TYPE_SOUNDRECORDING, "sound" },
-	{ TYPE_PERFORMANCE, "rehersal" },
-	{ TYPE_INTERNETSITE, "web site" },
-	{ TYPE_INTERVIEW, "interview" },
-	{ TYPE_INTERVIEW, "communication" },
-	{ TYPE_MISC, "misc" },
+	{ TYPE_FILM,             "videorecording" },
+	{ TYPE_FILM,             "motion picture" },
+	{ TYPE_SOUNDRECORDING,   "sound" },
+	{ TYPE_PERFORMANCE,      "rehersal" },
+	{ TYPE_INTERNETSITE,     "web site" },
+	{ TYPE_INTERVIEW,        "interview" },
+	{ TYPE_INTERVIEW,        "communication" },
+	{ TYPE_MISC,             "misc" },
 };
 int ngenres = sizeof( genres ) / sizeof( genres[0] );
 
@@ -193,17 +228,17 @@ static int
 get_type_from_genre( fields *info )
 {
 	int type = TYPE_UNKNOWN, i, j, level;
-	char *genre;
+	const char *genre, *tag;
 	for ( i=0; i<info->n; ++i ) {
-		if ( strcasecmp( info->tag[i].data, "GENRE" ) &&
-			strcasecmp( info->tag[i].data, "NGENRE" ) ) continue;
-		genre = info->data[i].data;
+		tag = (const char *) fields_tag( info, i, FIELDS_CHRP );
+		if ( strcasecmp( tag, "GENRE:MARC" ) && strcasecmp( tag, "GENRE:BIBUTILS" ) && strcasecmp( tag, "GENRE:UNKNOWN" ) ) continue;
+		genre = (const char *) fields_value( info, i, FIELDS_CHRP );
 		for ( j=0; j<ngenres; ++j ) {
 			if ( !strcasecmp( genres[j].out, genre ) )
 				type = genres[j].value;
 		}
 		if ( type==TYPE_UNKNOWN ) {
-			level = info->level[i];
+			level = fields_level( info, i );
 			if ( !strcasecmp( genre, "academic journal" ) ) {
 				type = TYPE_JOURNALARTICLE;
 			}
@@ -213,12 +248,12 @@ get_type_from_genre( fields *info )
 			}
 			else if ( !strcasecmp( genre, "book" ) ||
 				!strcasecmp( genre, "collection" ) ) {
-				if ( info->level[i]==0 ) type = TYPE_BOOK;
+				if ( fields_level( info, i ) == 0 ) type = TYPE_BOOK;
 				else type = TYPE_BOOKSECTION;
 			}
 			else if ( !strcasecmp( genre, "conference publication" ) ) {
 				if ( level==0 ) type=TYPE_CONFERENCE;
-				type = TYPE_PROCEEDINGS;
+				else type = TYPE_PROCEEDINGS;
 			}
 			else if ( !strcasecmp( genre, "thesis" ) ) {
 	                        if ( type==TYPE_UNKNOWN ) type=TYPE_THESIS;
@@ -238,11 +273,11 @@ static int
 get_type_from_resource( fields *info )
 {
 	int type = TYPE_UNKNOWN, i;
-	char *resource;
+	char *tag, *resource;
 	for ( i=0; i<info->n; ++i ) {
-		if ( strcasecmp( info->tag[i].data, "GENRE" )!=0 &&
-			strcasecmp( info->tag[i].data, "NGENRE" )!=0 ) continue;
-		resource = info->data[i].data;
+		tag = (char *) fields_tag( info, i, FIELDS_CHRP );
+		if ( strcasecmp( tag, "RESOURCE" ) ) continue;
+		resource = (char *) fields_value( info, i, FIELDS_CHRP );
 		if ( !strcasecmp( resource, "moving image" ) )
 			type = TYPE_FILM;
 	}
@@ -260,47 +295,72 @@ get_type( fields *info )
 }
 
 static void
-output_titleinfo( fields *info, FILE *outptr, char *tag, int level )
+output_titlebits( const char *mainttl, const char *subttl, FILE *outptr )
+{
+	if ( mainttl ) fprintf( outptr, "%s", mainttl );
+	if ( subttl ) {
+		if ( mainttl ) {
+			if ( mainttl[ strlen( mainttl ) - 1 ] != '?' )
+				fprintf( outptr, ": " );
+			else fprintf( outptr, " " );
+		}
+		fprintf( outptr, "%s", subttl );
+	}
+}
+
+static void
+output_titleinfo( const char *mainttl, const char *subttl, FILE *outptr, const char *tag, int level )
 {
-	newstr *mainttl = fields_findv( info, level, FIELDS_STRP, "TITLE" );
-	newstr *subttl  = fields_findv( info, level, FIELDS_STRP, "SUBTITLE" );
 	if ( mainttl || subttl ) {
 		fprintf( outptr, "<%s>", tag );
-		if ( mainttl ) fprintf( outptr, "%s", mainttl->data );
-		if ( subttl ) {
-			if ( mainttl ) {
-				if ( mainttl->len > 0 &&
-				     mainttl->data[mainttl->len-1]!='?' )
-					fprintf( outptr, ": " );
-				else fprintf( outptr, " " );
-			}
-			fprintf( outptr, "%s", subttl->data );
-		}
+		output_titlebits( mainttl, subttl, outptr );
 		fprintf( outptr, "</%s>\n", tag );
 	}
 }
 
 static void
-output_title( fields *info, FILE *outptr, int level )
+output_generaltitle( fields *info, FILE *outptr, const char *tag, int level )
 {
-	char *ttl    = fields_findv( info, level, FIELDS_CHRP, "TITLE" );
-	char *subttl = fields_findv( info, level, FIELDS_CHRP, "SUBTITLE" );
-	char *shrttl = fields_findv( info, level, FIELDS_CHRP, "SHORTTITLE" );
+	const char *ttl       = fields_findv( info, level, FIELDS_CHRP, "TITLE" );
+	const char *subttl    = fields_findv( info, level, FIELDS_CHRP, "SUBTITLE" );
+	const char *shrttl    = fields_findv( info, level, FIELDS_CHRP, "SHORTTITLE" );
+	const char *shrsubttl = fields_findv( info, level, FIELDS_CHRP, "SHORTSUBTITLE" );
 
-	output_titleinfo( info, outptr, "b:Title", level );
-
-	/* output shorttitle if it's different from normal title */
-	if ( shrttl ) {
-		if ( !ttl || ( strcmp( shrttl, ttl ) || subttl ) ) {
-			fprintf( outptr,  " <b:ShortTitle>" );
-			fprintf( outptr, "%s", shrttl );
-			fprintf( outptr, "</b:ShortTitle>\n" );
-		}
+	if ( ttl ) {
+		output_titleinfo( ttl, subttl, outptr, tag, level );
+	}
+	else if ( shrttl ) {
+		output_titleinfo( shrttl, shrsubttl, outptr, tag, level );
 	}
 }
 
 static void
-output_name_nomangle( FILE *outptr, char *p )
+output_maintitle( fields *info, FILE *outptr, int level )
+{
+	const char *ttl       = fields_findv( info, level, FIELDS_CHRP, "TITLE" );
+	const char *subttl    = fields_findv( info, level, FIELDS_CHRP, "SUBTITLE" );
+	const char *shrttl    = fields_findv( info, level, FIELDS_CHRP, "SHORTTITLE" );
+	const char *shrsubttl = fields_findv( info, level, FIELDS_CHRP, "SHORTSUBTITLE" );
+
+	if ( ttl ) {
+		output_titleinfo( ttl, subttl, outptr, "b:Title", level );
+
+		/* output shorttitle if it's different from normal title */
+		if ( shrttl ) {
+			if ( !ttl || ( strcmp( shrttl, ttl ) || subttl ) ) {
+				fprintf( outptr,  " <b:ShortTitle>" );
+				output_titlebits( shrttl, shrsubttl, outptr );
+				fprintf( outptr, "</b:ShortTitle>\n" );
+			}
+		}
+	}
+	else if ( shrttl ) {
+		output_titleinfo( shrttl, shrsubttl, outptr, "b:Title", level );
+	}
+}
+
+static void
+output_name_nomangle( FILE *outptr, const char *p )
 {
 	fprintf( outptr, "<b:Person>" );
 	fprintf( outptr, "<b:Last>%s</b:Last>", p );
@@ -308,42 +368,41 @@ output_name_nomangle( FILE *outptr, char *p )
 }
 
 static void
-output_name( FILE *outptr, char *p )
+output_name( FILE *outptr, const char *p )
 {
-	newstr family, part;
+	str family, part;
 	int n=0, npart=0;
 
-	newstr_init( &family );
-	while ( *p && *p!='|' ) newstr_addchar( &family, *p++ );
+	str_init( &family );
+	while ( *p && *p!='|' ) str_addchar( &family, *p++ );
 	if ( *p=='|' ) p++;
-	if ( family.len ) {
+	if ( str_has_value( &family ) ) {
 		fprintf( outptr, "<b:Person>" );
-		fprintf( outptr, "<b:Last>%s</b:Last>",family.data );
+		fprintf( outptr, "<b:Last>%s</b:Last>", str_cstr( &family ) );
 		n++;
 	}
-	newstr_free( &family );
+	str_free( &family );
 
-	newstr_init( &part );
+	str_init( &part );
 	while ( *p ) {
-		while ( *p && *p!='|' ) newstr_addchar( &part, *p++ );
-		if ( part.len ) {
+		while ( *p && *p!='|' ) str_addchar( &part, *p++ );
+		if ( str_has_value( &part ) ) {
 			if ( n==0 ) fprintf( outptr, "<b:Person>" );
 			if ( npart==0 ) 
-				fprintf( outptr, "<b:First>%s</b:First>",
-					part.data );
-			else fprintf( outptr, "<b:Middle>%s</b:Middle>",
-					part.data );
+				fprintf( outptr, "<b:First>%s</b:First>", str_cstr( &part ) );
+			else
+				fprintf( outptr, "<b:Middle>%s</b:Middle>", str_cstr( &part ) );
 			n++;
 			npart++;
 		}
 		if ( *p=='|' ) {
 			p++;
-			newstr_empty( &part );
+			str_empty( &part );
 		}
 	}
 	if ( n ) fprintf( outptr, "</b:Person>\n" );
 
-	newstr_free( &part );
+	str_free( &part );
 }
 
 
@@ -352,38 +411,36 @@ output_name( FILE *outptr, char *p )
 #define NAME_CORP (4)
 
 static int
-extract_name_and_info( newstr *outtag, newstr *intag )
+extract_name_and_info( str *outtag, str *intag )
 {
 	int code = NAME;
-	newstr_newstrcpy( outtag, intag );
-	if ( newstr_findreplace( outtag, ":ASIS", "" ) ) code = NAME_ASIS;
-	if ( newstr_findreplace( outtag, ":CORP", "" ) ) code = NAME_CORP;
+	str_strcpy( outtag, intag );
+	if ( str_findreplace( outtag, ":ASIS", "" ) ) code = NAME_ASIS;
+	if ( str_findreplace( outtag, ":CORP", "" ) ) code = NAME_CORP;
 	return code;
 }
 
 static void
-output_name_type( fields *info, FILE *outptr, int level, 
-			char *map[], int nmap, char *tag )
+output_name_type( fields *info, FILE *outptr, int level, char *map[], int nmap, const char *tag )
 {
-	newstr ntag;
+	str ntag;
 	int i, j, n=0, code, nfields;
-	newstr_init( &ntag );
+	str_init( &ntag );
 	nfields = fields_num( info );
 	for ( j=0; j<nmap; ++j ) {
 		for ( i=0; i<nfields; ++i ) {
-			code = extract_name_and_info( &ntag, &(info->tag[i]) );
-			if ( strcasecmp( ntag.data, map[j] ) ) continue;
+			code = extract_name_and_info( &ntag, fields_tag( info, i, FIELDS_STRP ) );
+			if ( strcasecmp( str_cstr( &ntag ), map[j] ) ) continue;
 			if ( n==0 )
 				fprintf( outptr, "<%s><b:NameList>\n", tag );
 			if ( code != NAME )
-				output_name_nomangle( outptr, info->data[i].data );
+				output_name_nomangle( outptr, (char *) fields_value( info, i, FIELDS_CHRP ) );
 			else 
-				output_name( outptr, info->data[i].data );
-			fields_setused( info, i );
+				output_name( outptr, (char *) fields_value( info, i, FIELDS_CHRP ) );
 			n++;
 		}
 	}
-	newstr_free( &ntag );
+	str_free( &ntag );
 	if ( n )
 		fprintf( outptr, "</b:NameList></%s>\n", tag );
 }
@@ -393,7 +450,7 @@ output_names( fields *info, FILE *outptr, int level, int type )
 {
 	char *authors[] = { "AUTHOR", "WRITER", "ASSIGNEE", "ARTIST",
 		"CARTOGRAPHER", "INVENTOR", "ORGANIZER", "DIRECTOR",
-		"PERFORMER", "REPORTER", "TRANSLATOR", "RECIPIENT",
+		"PERFORMER", "REPORTER", "TRANSLATOR", "ADDRESSEE",
 		"2ND_AUTHOR", "3RD_AUTHOR", "SUB_AUTHOR", "COMMITTEE",
 		"COURT", "LEGISLATIVEBODY" };
 	int nauthors = sizeof( authors ) / sizeof( authors[0] );
@@ -415,23 +472,28 @@ output_names( fields *info, FILE *outptr, int level, int type )
 static void
 output_date( fields *info, FILE *outptr, int level )
 {
-	char *year  = fields_findv_firstof( info, level, FIELDS_CHRP,
-			"PARTYEAR", "YEAR", NULL );
-	char *month = fields_findv_firstof( info, level, FIELDS_CHRP,
-			"PARTMONTH", "MONTH", NULL );
-	char *day   = fields_findv_firstof( info, level, FIELDS_CHRP,
-			"PARTDAY", "DAY", NULL );
+	const char *use;
+
+	const char *year  = fields_findv_firstof( info, level, FIELDS_CHRP,
+			"PARTDATE:YEAR", "DATE:YEAR", NULL );
+	const char *month = fields_findv_firstof( info, level, FIELDS_CHRP,
+			"PARTDATE:MONTH", "DATE:MONTH", NULL );
+	const char *day   = fields_findv_firstof( info, level, FIELDS_CHRP,
+			"PARTDATE:DAY", "DATE:DAY", NULL );
 	if ( year )  output_itemv( outptr, "b:Year", year, 0 );
-	if ( month ) output_itemv( outptr, "b:Month", month, 0 );
+	if ( month ) {
+		(void) number_to_full_month( month, &use );
+		output_itemv( outptr, "b:Month", use, 0 );
+	}
 	if ( day )   output_itemv( outptr, "b:Day", day, 0 );
 }
 
 static void
 output_pages( fields *info, FILE *outptr, int level )
 {
-	char *sn = fields_findv( info, LEVEL_ANY, FIELDS_CHRP, "PAGESTART" );
-	char *en = fields_findv( info, LEVEL_ANY, FIELDS_CHRP, "PAGEEND" );
-	char *ar = fields_findv( info, LEVEL_ANY, FIELDS_CHRP, "ARTICLENUMBER" );
+	const char *sn = fields_findv( info, LEVEL_ANY, FIELDS_CHRP, "PAGES:START" );
+	const char *en = fields_findv( info, LEVEL_ANY, FIELDS_CHRP, "PAGES:STOP" );
+	const char *ar = fields_findv( info, LEVEL_ANY, FIELDS_CHRP, "ARTICLENUMBER" );
 	if ( sn || en )
 		output_range( outptr, "b:Pages", sn, en, level );
 	else if ( ar )
@@ -442,13 +504,13 @@ static void
 output_includedin( fields *info, FILE *outptr, int type )
 {
 	if ( type==TYPE_JOURNALARTICLE ) {
-		output_titleinfo( info, outptr, "b:JournalName", 1 );
+		output_generaltitle( info, outptr, "b:JournalName", 1 );
 	} else if ( type==TYPE_ARTICLEINAPERIODICAL ) {
-		output_titleinfo( info, outptr, "b:PeriodicalTitle", 1 );
+		output_generaltitle( info, outptr, "b:PeriodicalTitle", 1 );
 	} else if ( type==TYPE_BOOKSECTION ) {
-		output_titleinfo( info, outptr, "b:ConferenceName", 1 ); /*??*/
+		output_generaltitle( info, outptr, "b:ConferenceName", 1 ); /*??*/
 	} else if ( type==TYPE_PROCEEDINGS ) {
-		output_titleinfo( info, outptr, "b:ConferenceName", 1 );
+		output_generaltitle( info, outptr, "b:ConferenceName", 1 );
 	}
 }
 
@@ -481,7 +543,7 @@ output_thesisdetails( fields *info, FILE *outptr, int type )
 			strcasecmp( tag, "DEGREEGRANTOR:ASIS") &
 			strcasecmp( tag, "DEGREEGRANTOR:CORP"))
 				continue;
-		output_item( info, outptr, "b:Institution", i, 0 );
+		output_item( info, outptr, "b:Institution", "", i, 0 );
 	}
 }
 
@@ -532,9 +594,9 @@ output_type( fields *info, FILE *outptr, int type )
 static void
 output_comments( fields *info, FILE *outptr, int level )
 {
+	const char *abs;
+	vplist_index i;
 	vplist notes;
-	char *abs;
-	int i;
 
 	vplist_init( &notes );
 
@@ -553,7 +615,7 @@ output_comments( fields *info, FILE *outptr, int level )
 static void
 output_bibkey( fields *info, FILE *outptr )
 {
-	char *bibkey = fields_findv_firstof( info, LEVEL_ANY, FIELDS_CHRP,
+	const char *bibkey = fields_findv_firstof( info, LEVEL_ANY, FIELDS_CHRP,
 			"REFNUM", "BIBKEY", NULL );
 	if ( bibkey ) output_itemv( outptr, "b:Tag", bibkey, 0 );
 }
@@ -562,19 +624,26 @@ static void
 output_citeparts( fields *info, FILE *outptr, int level, int max, int type )
 {
 	convert origin[] = {
-		{ "ADDRESS",	"b:City",	LEVEL_ANY },
-		{ "PUBLISHER",	"b:Publisher",	LEVEL_ANY },
-		{ "EDITION",	"b:Edition",	LEVEL_ANY }
+		{ "ADDRESS",	"b:City",	"", LEVEL_ANY },
+		{ "PUBLISHER",	"b:Publisher",	"", LEVEL_ANY },
+		{ "EDITION",	"b:Edition",	"", LEVEL_ANY }
 	};
 	int norigin = sizeof( origin ) / sizeof ( convert );
 	
 	convert parts[] = {
-		{ "VOLUME",          "b:Volume",  LEVEL_ANY },
-		{ "SECTION",         "b:Section", LEVEL_ANY },
-		{ "ISSUE",           "b:Issue",   LEVEL_ANY },
-		{ "NUMBER",          "b:Issue",   LEVEL_ANY },
-		{ "PUBLICLAWNUMBER", "b:Volume",  LEVEL_ANY },
-		{ "SESSION",         "b:Issue",   LEVEL_ANY },
+		{ "VOLUME",          "b:Volume",  "", LEVEL_ANY },
+		{ "SECTION",         "b:Section", "", LEVEL_ANY },
+		{ "ISSUE",           "b:Issue",   "", LEVEL_ANY },
+		{ "NUMBER",          "b:Issue",   "", LEVEL_ANY },
+		{ "PUBLICLAWNUMBER", "b:Volume",  "", LEVEL_ANY },
+		{ "SESSION",         "b:Issue",   "", LEVEL_ANY },
+		{ "URL",             "b:Url",     "", LEVEL_ANY },
+		{ "JSTOR",           "b:Url",     "http://www.jstor.org/stable/", LEVEL_ANY },
+		{ "ARXIV",           "b:Url",     "http://arxiv.org/abs/",        LEVEL_ANY },
+		{ "PMID",            "b:Url",     "http://www.ncbi.nlm.nih.gov/pubmed/", LEVEL_ANY },
+		{ "PMC",             "b:Url",     "http://www.ncbi.nlm.nih.gov/pmc/articles/", LEVEL_ANY },
+		{ "DOI",             "b:Url",     "https://doi.org/", LEVEL_ANY },
+		{ "MRNUMBER",        "b:Url",     "http://www.ams.org/mathscinet-getitem?mr=", LEVEL_ANY },
 	};
 	int nparts=sizeof(parts)/sizeof(convert);
 	
@@ -586,11 +655,11 @@ output_citeparts( fields *info, FILE *outptr, int level, int max, int type )
 	output_list( info, outptr, parts, nparts );
 	output_pages( info, outptr, level );
 	output_names( info, outptr, level, type );
-	output_title( info, outptr, 0 );
+	output_maintitle( info, outptr, 0 );
 	output_comments( info, outptr, level );
 }
 
-void
+static int
 wordout_write( fields *info, FILE *outptr, param *p, unsigned long numrefs )
 {
 	int max = fields_maxlevel( info );
@@ -601,9 +670,15 @@ wordout_write( fields *info, FILE *outptr, param *p, unsigned long numrefs )
 	fprintf( outptr, "</b:Source>\n" );
 
 	fflush( outptr );
+
+	return BIBL_OK;
 }
 
-void
+/*****************************************************
+ PUBLIC: void wordout_writeheader()
+*****************************************************/
+
+static void
 wordout_writeheader( FILE *outptr, param *p )
 {
 	if ( p->utf8bom ) utf8_writebom( outptr );
@@ -613,7 +688,11 @@ wordout_writeheader( FILE *outptr, param *p )
 		" xmlns=\"http://schemas.openxmlformats.org/officeDocument/2006/bibliography\" >\n");
 }
 
-void
+/*****************************************************
+ PUBLIC: void wordout_writefooter()
+*****************************************************/
+
+static void
 wordout_writefooter( FILE *outptr )
 {
 	fprintf(outptr,"</b:Sources>\n");
